from __future__ import annotations

from typing import Generator

from ..constants import API_VERSION
from ..schemas.conversations import (
    ConversationCreate,
    ConversationRead,
    ConversationReadFull,
    ConversationUpdate,
)
from ..schemas.feedback import FeedbackRead
from ..schemas.messages import AgentMessage, MessageRead, UserMessage
from ..schemas.runs import RunRead
from ..schemas.tags import TagRead
from .base import BaseResource


class ConversationsResource(BaseResource):
    _API_PREFIX: str = f"/{API_VERSION}/conversations"

    def create(
        self, *, conversation: ConversationCreate | None = None
    ) -> ConversationReadFull:
        """
        Creates a new conversation and get new conversation read full object,
        including the conversation_id.

        Args:
          conversation: This can be useful for including additional context about the conversation.

              If specified, any data included in the `data` field will be included as metadata within
              the conversation. We recommend structuring this metadata as a JSONencodable dictionary.

              If specified, the `source` field can be used as the source of the conversation.
              For instance, a particular chat widget.
        """
        if conversation is None:
            conversation = ConversationCreate()
        response = self.client.post(self._API_PREFIX, content=conversation)
        return ConversationReadFull(**response)

    def clone_conversation(
        self,
        *,
        message_id: str,
        conversation: ConversationUpdate | None = None,
    ) -> ConversationReadFull:
        """
        Clones a new conversation from a full or partial existing conversation. This works by
        cloning an existing conversation up to and including the message with the specified
        `message_id`. The new conversation will have a new `conversation_id`, which is different
        from the existing one.

        Args:
          message_id: The ID of the agent message up to which to clone an existing conversation.
              The system uses the agent message ID to identify a particular run step in the
              conversation.

          conversation: This can be useful for including additional context about the conversation.

              If specified, any data included in the `data` field will be included as
              metadata within the new conversation and will replace any metadata on the cloned
              conversation. We recommend structuring this metadata as a JSON encodable dictionary.

              If specified, the `source` field can be used as the source of the conversation.
              For instance, a particular chat widget.

              If `conversation` is not specified or set to None, and the cloned conversation
              already has metadata, then that metadata will be carried over to the new conversation.
        """
        if conversation is None:
            conversation = ConversationUpdate()
        query_params = {"message_id": message_id}
        response = self.client.post(
            f"{self._API_PREFIX}/clone",
            content=conversation,
            params=query_params,
        )
        return ConversationReadFull(**response)

    def search_own_conversations(
        self,
        *,
        tag_id: str | None = None,
        reversed: bool = True,
        limit: int = 20,
        offset: int = 0,
    ) -> list[ConversationRead]:
        """
        Fetches metadata on user's past conversations within the organization.
        If a user's API key is shared by multiple end users, then it will return conversations
        across all those end users.

        By default, it returns the most recent conversations first. It can additionally be
        filtered by `tag_id`.

        Args:
          tag_id: If specified, only conversations with the specified tag will be returned.

          reversed: If True, returns the most recent conversations first. Otherwise, returns the
              oldest conversations first.

          limit: Number of conversations to return.

          offset: Number of conversations to skip.
        """
        query_params = {
            "reversed": reversed,
            "limit": limit,
            "offset": offset,
        }
        if tag_id:
            query_params["tag_id"] = tag_id
        response = self.client.get(f"{self._API_PREFIX}/me", params=query_params)
        return [ConversationRead(**conversation_dict) for conversation_dict in response]

    def search_current_org_conversations(
        self,
        *,
        tag_id: str | None = None,
        reversed: bool = True,
        limit: int = 20,
        offset: int = 0,
    ) -> list[ConversationRead]:
        """
        Fetches metadata on all past conversations within the organization.

        By default, it returns the most recent conversations first. It can additionally be
        filtered by `tag_id`.

        Args:
          tag_id: If specified, only conversations with the specified tag will be returned.

          reversed: If True, returns the most recent conversations first. Otherwise, returns the
              oldest conversations first.

          limit: Number of conversations to return.

          offset: Number of conversations to skip.
        """
        query_params = {
            "reversed": reversed,
            "limit": limit,
            "offset": offset,
        }
        if tag_id:
            query_params["tag_id"] = tag_id
        response = self.client.get(f"{self._API_PREFIX}/org", params=query_params)
        return [ConversationRead(**conversation_dict) for conversation_dict in response]

    def search_saved_conversations(
        self, *, reversed: bool = True, limit: int = 20, offset: int = 0
    ) -> list[ConversationRead]:
        """
        Fetches metadata on user's saved conversations within the organization. Saved conversations
        can be any conversations carried out within the organization, not only the user's own.

        By default, it returns the most recent conversations first.

        Args:
          reversed: If True, returns the most recent conversations first. Otherwise, returns the
              oldest conversations first.

          limit: Number of conversations to return.

          offset: Number of conversations to skip.
        """
        query_params = {
            "reversed": reversed,
            "limit": limit,
            "offset": offset,
        }
        response = self.client.get(f"{self._API_PREFIX}/saved", params=query_params)
        return [ConversationRead(**conversation_dict) for conversation_dict in response]

    def get_conversation_metadata(
        self, *, conversation_id: str
    ) -> ConversationReadFull:
        """
        Fetches metadata on a specified conversation.

        Args:
          conversation_id: The ID of the conversation for which to get the metadata.
        """
        response = self.client.get(f"{self._API_PREFIX}/{conversation_id}")
        return ConversationReadFull(**response)

    def update_conversation_metadata(
        self, *, conversation_id: str, conversation: ConversationUpdate
    ) -> ConversationReadFull:
        """
        Updates metadata on a specified conversation.

        Args:
          conversation_id: The ID of the conversation for which to update the metadata.

          conversation: The updated conversation metadata.
        """
        response = self.client.patch(
            f"{self._API_PREFIX}/{conversation_id}",
            content=conversation,
        )
        return ConversationReadFull(**response)

    def save_conversation(self, *, conversation_id: str) -> ConversationReadFull:
        """
        Adds a specified conversation to user's saved conversations list. The conversation can
        be any conversation carried out within the organization, not only the user's own. The
        user is identified based on the API key being used.

        Args:
          conversation_id: The ID of the conversation to save.
        """
        response = self.client.post(f"{self._API_PREFIX}/{conversation_id}/saved")
        return ConversationReadFull(**response)

    def unsave_conversation(self, *, conversation_id: str) -> ConversationReadFull:
        """
        Removes a specified conversation from user's saved conversations list.
        This does not delete the underlying conversation and only removes a reference
        to it in the user's list. The user is identified based on the API key being used.

        Args:
          conversation_id: The ID of the conversation to remove.
        """
        response = self.client.delete(f"{self._API_PREFIX}/{conversation_id}/saved")
        return ConversationReadFull(**response)

    def add_messages_to_conversation_history(
        self, *, conversation_id: str, messages: list[UserMessage | AgentMessage]
    ) -> list[MessageRead]:
        """
        Adds messages to a specified conversation's message history without triggering an agent
        response. Messages are added in the order in which they are provided.
        """
        response = self.client.post(
            f"{self._API_PREFIX}/{conversation_id}/messages",
            content=[message.model_dump() for message in messages],
        )
        return [MessageRead(**message_dict) for message_dict in response]

    def get_conversation_message_history(
        self, *, conversation_id: str
    ) -> list[MessageRead]:
        """
        Fetches the full message history for a specified conversation.

        Args:
          conversation_id: The ID of the conversation for which to get the message history.
        """
        response = self.client.get(f"{self._API_PREFIX}/{conversation_id}/messages")
        return [MessageRead(**message_dict) for message_dict in response]

    def get_conversation_feedback(self, *, conversation_id: str) -> list[FeedbackRead]:
        """
        Fetches all feedback submitted for a specified conversation.

        Args:
          conversation_id: The ID of the conversation for which to get the feedback.
        """
        response = self.client.get(f"{self._API_PREFIX}/{conversation_id}/feedback")
        return [FeedbackRead(**feedback_dict) for feedback_dict in response]

    def get_conversation_runs(self, *, conversation_id: str) -> list[RunRead]:
        """
        Fetches the information on individual runs within a specified conversation. Each run
        represent a particular step in the conversation - one interaction between a user and
        the agent.

        Args:
          conversation_id: The ID of the conversation for which to get the runs.
        """
        response = self.client.get(f"{self._API_PREFIX}/{conversation_id}/runs")
        return [RunRead(**run_dict) for run_dict in response]

    def get_conversation_tags(self, *, conversation_id: str) -> list[TagRead]:
        """
        Fetches all tags associated with a specified conversation.

        Args:
          conversation_id: The ID of the conversation for which to get the tags.
        """
        response = self.client.get(f"{self._API_PREFIX}/{conversation_id}/tags")
        return [TagRead(**tag_dict) for tag_dict in response]

    def add_tag_to_conversation(
        self, *, conversation_id: str, tag_id: str
    ) -> list[TagRead]:
        """
        Adds a tag to a specified conversation. Returns all tags associated with the conversation.

        Args:
          conversation_id: The ID of the conversation to tag.

          tag_id: The ID of the tag to add.
        """
        response = self.client.post(
            f"{self._API_PREFIX}/{conversation_id}/tags/{tag_id}"
        )
        return [TagRead(**tag_dict) for tag_dict in response]

    def remove_tag_from_conversation(
        self, *, conversation_id: str, tag_id: str
    ) -> list[TagRead]:
        """
        Removes a tag from a specified conversation. Returns all tags associated with the
        conversation.

        Args:
          conversation_id: The ID of the conversation to untag.

          tag_id: The ID of the tag to remove.
        """
        response = self.client.delete(
            f"{self._API_PREFIX}/{conversation_id}/tags/{tag_id}"
        )
        return [TagRead(**tag_dict) for tag_dict in response]

    def chat_with_agent(
        self,
        *,
        conversation_id: str,
        messages: list[UserMessage],
        agent: str,
        stream: bool = False,
        return_inputs: bool = False,
    ) -> list[MessageRead] | Generator[MessageRead]:
        """
        Posts new message(s) to the agent on this conversation and gets a list of messages as a response.
        For most use cases, the new message will be a single user message. Messages sent with the request
        will be saved to the conversation history and then used by the agent.

        If sending more than one message, the list of messages must be in ascending order, from oldest to most
        recent. Be careful not to include any messages multiple times otherwise the agent will see multiple copies of
        messages and may be confused.

        If called with an empty message list in the request `[]` then the agent will be called with no messages.
        The agent will only respond if the message most recently saved to the conversation history was a user message.

        If `stream=True` the response will be streamed as an event stream of status and agent messages.
        Messages can be accessed by iterating over the response.

        Args:
          conversation_id: The ID of the conversation to post messages to.

          messages: List of messages to send to the agent.

          agent: The name of the agent to send the message to.

          stream: If True, the response will be streamed as an event stream of status and agent messages.

          return_inputs: If True, returns the user inputs along with the agent response. This can be useful if you need
              access to the user message IDs for the
               conversation history.
        """
        query_params = {"agent": agent}
        messages = [message.model_dump() for message in messages]

        if return_inputs is True:
            query_params["return_inputs"] = return_inputs

        if stream is True:
            query_params["stream"] = stream
            iterator = self.client.post_stream(
                f"{self._API_PREFIX}/{conversation_id}/chat",
                params=query_params,
                content={"messages": messages},
            )

            return (MessageRead(**message) for message in iterator)

        else:
            response = self.client.post(
                f"{self._API_PREFIX}/{conversation_id}/chat",
                params=query_params,
                content={"messages": messages},
            )

            return [MessageRead(**message) for message in response]



from pydantic import BaseModel


class TagRead(BaseModel):
    id: str
    org_id: str
    name: str
    description: str


class TagCreate(BaseModel):
    name: str
    description: str = ""


class TagUpdate(BaseModel):
    description: str | None = None



from json import JSONDecodeError

import httpx

from .errors import (
    AlreadyExistsApiError,
    BadRequestApiError,
    ForbiddenApiError,
    GenericApiError,
    HttpResponseError,
    MethodNotAllowedApiError,
    NotFoundApiError,
    UnauthorizedApiError,
    UnprocessableEntityApiError,
)


def handle_response_error(response: httpx.Response, parse_response: bool = True):
    try:
        response_content = response.json() if parse_response else {}
    except JSONDecodeError:
        response_content = {"detail": response.text}

    error_detail = response_content.get("detail")
    if not isinstance(error_detail, dict):  # normalize detail if not data structure
        error_detail = {"response": error_detail}

    error_args = error_detail if error_detail else response_content

    if response.status_code == BadRequestApiError.HTTP_STATUS:
        error_type = BadRequestApiError
    elif response.status_code == UnauthorizedApiError.HTTP_STATUS:
        error_type = UnauthorizedApiError
    elif response.status_code == AlreadyExistsApiError.HTTP_STATUS:
        error_type = AlreadyExistsApiError
    elif response.status_code == ForbiddenApiError.HTTP_STATUS:
        error_type = ForbiddenApiError
    elif response.status_code == NotFoundApiError.HTTP_STATUS:
        error_type = NotFoundApiError
    elif response.status_code == UnprocessableEntityApiError.HTTP_STATUS:
        error_type = UnprocessableEntityApiError
    elif response.status_code == MethodNotAllowedApiError.HTTP_STATUS:
        error_type = MethodNotAllowedApiError
    elif response.status_code == GenericApiError.HTTP_STATUS:
        error_type = GenericApiError
    else:
        raise HttpResponseError(response=response)
    raise error_type(**error_args)



import json
from typing import Any

import pydantic

from .pydantic import model_json


def prepare_content(content: Any) -> str:
    SEPARATORS = (",", ":")
    if isinstance(content, str):
        pass

    elif isinstance(content, pydantic.BaseModel):
        content = model_json(content)

    elif isinstance(content, list):
        updated_content = []
        for c in content:
            if isinstance(c, pydantic.BaseModel):
                updated_content.append(json.loads(model_json(c)))
            else:
                updated_content.append(c)
        content = json.dumps(updated_content, separators=SEPARATORS)

    else:
        content = json.dumps(content, separators=SEPARATORS)

    return content



from datetime import datetime

from pydantic import BaseModel, Field

from .tags import TagRead


class ConversationRead(BaseModel):
    id: str
    org_id: str
    data: dict
    source: str = Field(..., description="Source of the conversation.")
    cloned_from_message_id: str | None
    api_auth: bool = Field(..., description="Was this conversation created by API key?")
    summary: str = Field(..., description="A short summary of the conversation.")
    created: datetime


class ConversationReadFull(BaseModel):
    id: str
    org_id: str
    data: dict
    source: str = Field(..., description="Source of the conversation.")
    cloned_from_message_id: str | None
    api_auth: bool = Field(..., description="Was this conversation created by API key?")
    summary: str = Field(..., description="A short summary of the conversation.")
    created: datetime
    user_id: str
    tags: list[TagRead] = Field(
        ..., description="List of active Tags on this conversation."
    )
    is_saved: bool = Field(
        ..., description="Is this conversation saved for the calling user?"
    )


class ConversationUpdate(BaseModel):
    data: dict | None = Field(None, description="Optional metadata.")
    source: str | None = Field(None, description="Optional source of the conversation.")


class ConversationCreate(BaseModel):
    data: dict = Field({}, description="Optional metadata.")
    source: str = Field("", description="Optional source of the conversation.")



from datetime import datetime
from typing import Literal

from pydantic import BaseModel, Field


class FeedbackRead(BaseModel):
    id: str
    org_id: str
    conversation_id: str
    run_id: str
    user_id: str
    thumbs: Literal["up", "down", ""] = Field(
        ..., description="Thumbs up or down feedback from user."
    )
    comment: str = Field(..., description="Optional comment from user.")
    problems: list[str] = Field(
        ..., description="List of problems identified by user in red teaming."
    )
    severity: str = Field(
        ..., description="Severity of the `problems` identified by user in red teaming."
    )
    effort: str = Field(
        ...,
        description="User's effort required to discover the `problems` in red teaming.",
    )
    description: str = Field(
        ...,
        description="Description of the `problems` identified by user in red teaming.",
    )
    created: datetime
    updated: datetime | None


class FeedbackUpsert(BaseModel):
    thumbs: Literal["up", "down", ""] | None = Field(
        None, description="Thumbs up or down feedback from user."
    )
    comment: str | None = Field(None, description="Optional comment from user.")
    problems: list[str] | None = Field(
        None, description="List of problems identified by user in red teaming."
    )
    severity: str | None = Field(
        None,
        description="Severity of the `problems` identified by user in red teaming.",
    )
    effort: str | None = Field(
        None,
        description="User's effort required to discover the `problems` in red teaming.",
    )
    description: str | None = Field(
        None,
        description="Description of the `problems` identified by user in red teaming.",
    )



import pydantic

PYDANTIC_V2 = pydantic.VERSION.startswith("2.")


def model_json(model: pydantic.BaseModel, *, indent: int | None = None) -> str:
    if PYDANTIC_V2:
        return model.model_dump_json(indent=indent)
    return model.json(indent=indent)  # type: ignore



from datetime import datetime
from typing import Literal

from pydantic import BaseModel


class MessageRead(BaseModel):
    """A message via the Engage Smarter API"""

    id: str
    org_id: str
    conversation_id: str
    run_id: str
    role: str
    content: str
    name: str | None = None
    user_id: str | None = None
    created: datetime


class MessageBasic(BaseModel):
    """Message with the basic fields - Not from the API"""

    role: str
    content: str
    name: str | None = None


class UserMessage(MessageBasic):
    """Message from a user"""

    role: Literal["user"] = "user"
    content: str
    name: str | None = None


class AgentMessage(MessageBasic):
    """Message from an agent to the user"""

    role: Literal["agent"] = "agent"
    content: str
    name: str | None = None


class InfoMessage(MessageBasic):
    """Message created by an agent that provides additional metadata"""

    role: Literal["info"] = "info"
    content: str
    name: str | None = None


class StatusMessage(MessageBasic):
    """Message from an agent that is emitted during an API call that provides a status update to the user."""

    role: Literal["status"] = "status"
    content: str
    name: str | None = None



from datetime import datetime

from pydantic import BaseModel

from .tags import TagRead


class RunRead(BaseModel):
    id: str
    org_id: str
    conversation_id: str
    agent: str
    test: bool
    api_auth: bool
    created: datetime


class RunReadFull(BaseModel):
    id: str
    org_id: str
    conversation_id: str
    agent: str
    tags: list[TagRead]
    test: bool
    api_auth: bool
    created: datetime



import json
from typing import Any

import httpx

from .constants import API_KEY_HEADER_NAME, ORG_ID_HEADER_NAME
from .errors import GenericApiError
from .errors_handler import handle_response_error
from .utils import is_mapping
from .utils.content import prepare_content
from .utils.streaming import SSEDecoder


class AuthenticatedClient:
    def __init__(
        self,
        api_url: str,
        api_key: str,
        org_id: str,
        timeout: int,
        headers: dict[str, str] | None = None,
    ):
        self.api_url = api_url
        self.api_key = api_key
        self.org_id = org_id
        self.timeout = timeout
        self._headers = headers or {}
        self._client = httpx.Client(
            base_url=self.api_url,
            headers=self.get_headers(),
            timeout=self.timeout,
        )

    def get_headers(self) -> dict[str, str]:
        """Get headers to be used in authenticated endpoints"""
        return {
            API_KEY_HEADER_NAME: self.api_key,
            ORG_ID_HEADER_NAME: self.org_id,
            **self._headers,
        }

    def _process_response(self, response: httpx.Response) -> Any:
        if not response.is_success:
            handle_response_error(response)
        return response.json()

    def get(self, url: str, **kwargs) -> Any:
        res = self._client.get(url, **kwargs)
        return self._process_response(res)

    def post(self, url: str, **kwargs) -> Any:
        if "content" in kwargs:
            content = kwargs["content"]
            kwargs["content"] = prepare_content(content)
        res = self._client.post(url, **kwargs)
        return self._process_response(res)

    def post_stream(self, url: str, **kwargs) -> Any:
        if "content" in kwargs:
            content = kwargs["content"]
            kwargs["content"] = prepare_content(content)

        with self._client.stream("POST", url, **kwargs) as res:
            if not res.is_success:
                handle_response_error(res, parse_response=False)

            decoder = SSEDecoder()
            iterator = decoder.iter(res.iter_lines())

            for sse in iterator:
                if sse.data.startswith("[DONE]"):
                    break

                if sse.event is None:
                    # TODO: Confirm shape of our data when there is an error
                    data = sse.json()
                    if is_mapping(data) and data.get("error"):
                        raise GenericApiError(
                            {"detail": "An error occurred while streaming."}
                        )

                    yield json.loads(sse.data)

    def patch(self, url: str, **kwargs) -> Any:
        if "content" in kwargs:
            content = kwargs["content"]
            kwargs["content"] = prepare_content(content)
        res = self._client.patch(url, **kwargs)
        return self._process_response(res)

    def delete(self, url: str, **kwargs) -> Any:
        res = self._client.delete(url, **kwargs)
        return self._process_response(res)

    def __del__(self):
        if hasattr(self, "_client"):
            del self._client

    # Context managers

    def close(self):
        self._client.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()



import os
import re
from typing import Dict

from ._auth_client import AuthenticatedClient
from .constants import API_VERSION, DEFAULT_API_URL, DEFAULT_TIMEOUT
from .errors import EngageSmarterError
from .resources.agents import AgentsResource
from .resources.conversations import ConversationsResource
from .resources.runs import RunsResource
from .resources.tags import TagsResource
from .schemas.base import Success


class Client:
    """
    The main synchronous Engage Smarter AI Platform client.
    """

    def __init__(
        self,
        *,
        api_url: str | None = None,
        api_key: str | None = None,
        org_id: str | None = None,
        timeout: int = DEFAULT_TIMEOUT,
        extra_headers: Dict[str, str] | None = None,
    ):
        """Construct a new synchronous client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `ENGAGE_SMARTER_API_KEY`
        - `org_id` from `ENGAGE_SMARTER_ORG_ID`
        """

        api_url = api_url or os.getenv("ENGAGE_SMARTER_API_URL")
        api_url = api_url or DEFAULT_API_URL
        # Checking that the api_url does not end in '/'
        api_url = re.sub(r"\/$", "", api_url)

        api_key = api_key or os.getenv("ENGAGE_SMARTER_API_KEY")
        if api_key is None:
            raise EngageSmarterError(
                "api_key is required to use the Engage Smarter AI Platform API. Either pass it as an argument or set the ENGAGE_SMARTER_API_KEY environment variable."
            )

        org_id = org_id or os.getenv("ENGAGE_SMARTER_ORG_ID")
        if org_id is None:
            raise EngageSmarterError(
                "org_id is required to use the Engage Smarter AI Platform API. Either pass it as an argument or set the ENGAGE_SMARTER_ORG_ID environment variable."
            )

        headers = extra_headers or {}

        self._client = AuthenticatedClient(
            api_url=api_url,
            api_key=api_key,
            org_id=org_id,
            timeout=timeout,
            headers=headers,
        )

    def validate(self) -> Success:
        """
        Validate credentials.
        """
        response = self._client.post(f"/{API_VERSION}/users/validate-me")
        return Success(**response)

    def _check_engagesmarter_versions(self):
        """
        Check that the Engage Smarter API version is compatible with the SDK version.
        """
        # TODO: versioning
        raise NotImplementedError

    @property
    def agents(self):
        """
        The agents endpoint.
        """
        return AgentsResource(client=self._client)

    @property
    def conversations(self):
        """
        The conversations endpoint.
        """
        return ConversationsResource(client=self._client)

    @property
    def runs(self):
        """
        The runs endpoint.
        """
        return RunsResource(client=self._client)

    @property
    def tags(self):
        """
        The tags endpoint.
        """
        return TagsResource(client=self._client)

    def __del__(self):
        if hasattr(self, "_client"):
            del self._client

    # Context managers

    def close(self):
        self._client.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()



import httpx


class EngageSmarterError(Exception):
    pass


class EsApiResponseError(EngageSmarterError):
    HTTP_STATUS: int

    def __init__(self, **ctx):
        self.ctx = ctx

    def __str__(self):
        return (
            f"Engage Smarter API returned an error with http status: {self.HTTP_STATUS}. "
            f"Error details: {self.ctx!r}"
        )


class BadRequestApiError(EsApiResponseError):
    HTTP_STATUS = 400


class UnauthorizedApiError(EsApiResponseError):
    HTTP_STATUS = 401


class ForbiddenApiError(EsApiResponseError):
    HTTP_STATUS = 403


class NotFoundApiError(EsApiResponseError):
    HTTP_STATUS = 404


class MethodNotAllowedApiError(EsApiResponseError):
    HTTP_STATUS = 405


class AlreadyExistsApiError(EsApiResponseError):
    HTTP_STATUS = 409


class UnprocessableEntityApiError(EsApiResponseError):
    HTTP_STATUS = 422


class GenericApiError(EsApiResponseError):
    HTTP_STATUS = 500


class HttpResponseError(EsApiResponseError):
    """Used to handle http errors other than those defined by the Engage Smarter API."""

    def __init__(self, response: httpx.Response):
        self.status_code = response.status_code
        self.detail = response.content
        self.HTTP_STATUS = self.status_code

    def __str__(self):
        return (
            "Received an HTTP error from server\n"
            + f"Response status: {self.status_code}\n"
            + f"Response content: {self.detail}\n"
        )



from ..constants import API_VERSION
from ..schemas.base import Success
from ..schemas.conversations import ConversationRead
from ..schemas.runs import RunRead
from ..schemas.tags import TagCreate, TagRead, TagUpdate
from .base import BaseResource


class TagsResource(BaseResource):
    _API_PREFIX: str = f"/{API_VERSION}/tags"

    def search(
        self,
        *,
        search: str = "",
        reversed: bool = True,
        limit: int = 20,
        offset: int = 0,
    ) -> list[TagRead]:
        """
        Searches for tags with names or descriptions matching the given search string.
        Only searches for tags available within the current organization.

        Args:
          reversed: If True, get most recently created tag first. True by default.

          search: A search string to search for among tag names and descriptions.

          limit: The maximum number of tags to return.

          offset: The number of tags to skip before returning results.
        """
        response = self.client.get(
            f"{self._API_PREFIX}",
            params={
                "search": search,
                "limit": limit,
                "offset": offset,
                "reversed": reversed,
            },
        )
        return [TagRead(**tag_dict) for tag_dict in response]

    def create(self, *, name: str, description: str = "") -> TagRead:
        """
        Creates a new tag in the current organization.

        Args:
          name: The name of the tag.

          description: A description of the tag. If not provided, defaults to an empty string.
        """
        content = TagCreate(name=name, description=description)
        response = self.client.post(f"{self._API_PREFIX}", content=content)
        return TagRead(**response)

    def get_tag(self, *, tag_id: str) -> TagRead:
        """
        Fetches a single tag by ID from within the current organization.

        Args:
          tag_id: The ID of the tag to fetch.
        """
        response = self.client.get(f"{self._API_PREFIX}/{tag_id}")
        return TagRead(**response)

    def update_tag(self, *, tag_id: str, description: str | None = None) -> TagRead:
        """
        Updates a single tag by ID from within the current organization.

        Args:
          tag_id: The ID of the tag to update.

          description: A description of the tag.
        """
        content = TagUpdate(description=description)
        response = self.client.patch(f"{self._API_PREFIX}/{tag_id}", content=content)
        return TagRead(**response)

    def delete_tag(self, *, tag_id: str) -> Success:
        """
        Deletes a single tag by ID from within the current organization.

        Args:
          tag_id: The ID of the tag to delete.
        """
        response = self.client.delete(f"{self._API_PREFIX}/{tag_id}")
        return Success(**response)

    def get_tagged_conversations(
        self, *, reversed: bool = True, tag_id: str, limit: int = 20, offset: int = 0
    ) -> list[ConversationRead]:
        """
        Fetches a list of metadata objects for conversations that have been linked to a given tag.

        Args:
          reversed: If True, get most recent conversation first. True by default.

          tag_id: The ID of the tag to fetch conversations for.

          limit: The maximum number of conversations to return.

          offset: The number of conversations to skip before returning results.
        """
        response = self.client.get(
            f"{self._API_PREFIX}/{tag_id}/conversations",
            params={"limit": limit, "offset": offset, "reversed": reversed},
        )
        return [ConversationRead(**conversation) for conversation in response]

    def get_tagged_runs(
        self, *, reversed: bool = True, tag_id: str, limit: int = 20, offset: int = 0
    ) -> list[RunRead]:
        """
        Fetches a list of metadata objects for runs that have been linked to a given tag.

        Args:
          reversed: If True, get most recent run first. True by default.

          tag_id: The ID of the tag to fetch runs for.

          limit: The maximum number of runs to return.

          offset: The number of runs to skip before returning results.
        """
        response = self.client.get(
            f"{self._API_PREFIX}/{tag_id}/runs",
            params={"limit": limit, "offset": offset, "reversed": reversed},
        )
        return [RunRead(**run) for run in response]



from enum import Enum

from pydantic import BaseModel, Field


class AgentMetadata(BaseModel):
    """Metadata about an agent on the Engage Smarter AI Platform API."""

    name: str = Field(..., description="Name and version of the agent.")
    title: str = Field(..., description="Human descriptive title for the agent.")
    description: str = Field(
        ..., description="Description providing more detail about the agent."
    )
    welcome_message: str = Field(
        "", description="Optional welcome message to be displayed to the user."
    )
    available_all_orgs: bool = Field(
        ...,
        description="Whether the agent is available to all orgs. False if available_org_ids is provided.",
    )
    available_org_ids: list[str] | None = Field(
        ...,
        description="List of org IDs this agent is available to. None if available_all_orgs is True.",
    )


class AgentStatusType(str, Enum):
    """Type of the agent status."""

    ACTIVE = "active"
    FAIL = "fail"


class AgentStatus(BaseModel):
    """Status of the agent."""

    status: AgentStatusType = Field(
        ...,
        description="Status of the agent. Can be one of 'active', 'fail'.",
    )



from typing import Iterator

from ..constants import API_VERSION
from ..schemas.feedback import FeedbackRead, FeedbackUpsert
from ..schemas.messages import AgentMessage, MessageRead, UserMessage
from ..schemas.runs import RunRead
from ..schemas.tags import TagRead
from .base import BaseResource


class RunsResource(BaseResource):
    _API_PREFIX: str = f"/{API_VERSION}/runs"

    def create_or_update_run_feedback(
        self, *, run_id: str, feedback: FeedbackUpsert
    ) -> FeedbackRead:
        """
        Creates or updates (upsert) run feedback.

        Feedback can comprise two fields: (1) a `thumbs` "up" or "down" rating, and (2) a free-text
        `comment`. If updating existing feedback by the user on a run, make sure to fill out whichever
        fields have previously been submitted, as otherwise the existing feedback will be overwritten.

        Args:
          run_id: The ID of the run to give feedback on.

          feedback: The user's feedback record to attach to the run.
        """
        response = self.client.post(
            f"{self._API_PREFIX}/{run_id}/feedback", content=feedback
        )
        return FeedbackRead(**response)

    def get_user_runs(
        self,
        *,
        tag_id: str | None = None,
        agent: str | None = None,
        reversed: bool = True,
        limit: int = 20,
        offset: int = 0,
    ) -> list[RunRead]:
        """
        Fetches a list of the user's run metadata records for the current organization.
        These records are created when a run is started, and contain information about
        the run, such as the agent used, the conversation ID, and the start time of the run.

        The list of runs will include runs which form part of conversations that the user
        has had via the API using an API key or in an app.

        Args:
          tag_id: Filter by tag ID.

          agent: Filter by agent.

          reversed: If True, get most recent run first. True by default.

          limit: The maximum number of runs to return.

          offset: The number of runs to skip before returning results.
        """
        query_params = {
            "reversed": reversed,
            "limit": limit,
            "offset": offset,
        }
        if tag_id is not None:
            query_params["tag_id"] = tag_id
        if agent is not None:
            query_params["agent"] = agent
        response = self.client.get(f"{self._API_PREFIX}/me", params=query_params)
        return [RunRead(**run) for run in response]

    def get_run_metadata(self, *, run_id: str) -> RunRead:
        """
        Fetches the metadata record for a single run. The run can be any within
        the user's current organization, not only the user's own runs.

        Args:
          run_id: The ID of the run to fetch.
        """
        response = self.client.get(f"{self._API_PREFIX}/{run_id}")
        return RunRead(**response)

    def get_run_messages(self, *, run_id: str) -> list[MessageRead]:
        """
        Fetches the messages for a single run. The run can be any within
        the user's current organization, not only the user's own runs.

        Args:
          run_id: The ID of the run to fetch.
        """
        response = self.client.get(f"{self._API_PREFIX}/{run_id}/messages")
        return [MessageRead(**message) for message in response]

    def get_run_feedback(self, *, run_id: str) -> list[FeedbackRead]:
        """
        Fetches the feedback for a single run. The run can be any within
        the user's current organization, not only the user's own runs.

        Args:
          run_id: The ID of the run to fetch.
        """
        response = self.client.get(f"{self._API_PREFIX}/{run_id}/feedback")
        return [FeedbackRead(**feedback) for feedback in response]

    def get_run_tags(self, *, run_id: str) -> list[TagRead]:
        """
        Fetches all tags associated with a specified run.

        Args:
          run_id: The ID of the run for which to get the tags.
        """
        response = self.client.get(f"{self._API_PREFIX}/{run_id}/tags")
        return [TagRead(**tag_dict) for tag_dict in response]

    def add_tag_to_run(self, *, run_id: str, tag_id: str) -> list[TagRead]:
        """
        Adds a tag to a specified run. Returns all tags associated with the run.

        Args:
          run_id: The ID of the run to tag.

          tag_id: The ID of the tag to add.
        """
        response = self.client.post(f"{self._API_PREFIX}/{run_id}/tags/{tag_id}")
        return [TagRead(**tag_dict) for tag_dict in response]

    def remove_tag_from_run(self, *, run_id: str, tag_id: str) -> list[TagRead]:
        """
        Removes a tag from a specified run. Returns all tags associated with the
        run.

        Args:
          run_id: The ID of the run to untag.

          tag_id: The ID of the tag to remove.
        """
        response = self.client.delete(f"{self._API_PREFIX}/{run_id}/tags/{tag_id}")
        return [TagRead(**tag_dict) for tag_dict in response]

    def chat_with_agent(
        self,
        *,
        messages: list[AgentMessage | UserMessage | MessageRead],
        agent: str,
        stream: bool = False,
        return_inputs: bool = False,
        data: dict = {},
    ) -> list[MessageRead] | Iterator[MessageRead]:
        """
        Creates a new run by posting a history of user and agent messages, and expecting an agent message in response.
        The list of messages must be in ascending order, from oldest to most recent.

        A new conversation ID will be generated automatically for this run and will be returned on the agent message.
        The conversation can be continued using the conversations API by passing in the conversation ID found on
        messages returned by this method.

        If `stream=True` the response will be streamed as an event stream of status and agent messages.
        Messages can be accessed by iterating over the response.

        Args:
          messages: List of messages to send to the agent.

          agent: The name of the agent to use. The agent will respond to the latest message in the history.

          stream: If True, the response will be streamed as an event stream of status and agent messages.

          return_inputs: If True, returns the user inputs along with the agent response. This can be useful if you need
              access to the user message IDs for the conversation history.

          data: A dictionary of additional metadata to send with the run. This can be accessed from the conversation to
              which the run belongs.
        """
        query_params = {"agent": agent}
        messages = [message.model_dump() for message in messages]

        if return_inputs is True:
            query_params["return_inputs"] = return_inputs

        if stream is True:
            query_params["stream"] = stream
            iterator = self.client.post_stream(
                f"{self._API_PREFIX}/chat",
                params=query_params,
                content={"messages": messages, "data": data},
            )

            return (MessageRead(**message) for message in iterator)

        else:
            response = self.client.post(
                f"{self._API_PREFIX}/chat",
                params=query_params,
                content={"messages": messages, "data": data},
            )

            return [MessageRead(**message) for message in response]



from ..constants import API_VERSION
from ..resources.base import BaseResource
from ..schemas.agents import AgentMetadata, AgentStatus


class AgentsResource(BaseResource):
    _API_PREFIX: str = f"/{API_VERSION}/agents"

    def list(self) -> list[AgentMetadata]:
        """
        Lists metadata for all agents available via the API.
        """
        response: dict = self.client.get(self._API_PREFIX)
        return [AgentMetadata(**agent_dict) for agent_dict in response.values()]

    def validate_agent(self, *, agent: str) -> AgentStatus:
        """
        Validates the name for the specified agent.
        """
        params = {"agent": agent}
        response: dict = self.client.get(f"{self._API_PREFIX}/validate", params=params)
        return AgentStatus(**response)



from importlib import metadata

try:
    __version__ = metadata.version(__package__)
except metadata.PackageNotFoundError:
    # Case where package metadata is not available.
    __version__ = ""

from .client import Client
from .schemas.agents import AgentMetadata
from .schemas.base import Success
from .schemas.conversations import (
    ConversationCreate,
    ConversationRead,
    ConversationReadFull,
    ConversationUpdate,
)
from .schemas.feedback import FeedbackRead, FeedbackUpsert
from .schemas.messages import (
    AgentMessage,
    InfoMessage,
    MessageBasic,
    MessageRead,
    StatusMessage,
    UserMessage,
)
from .schemas.runs import RunRead, RunReadFull
from .schemas.tags import TagCreate, TagRead, TagUpdate

__all__ = [
    Client,
    AgentMessage,
    AgentMetadata,
    ConversationCreate,
    ConversationRead,
    ConversationReadFull,
    ConversationUpdate,
    FeedbackRead,
    FeedbackUpsert,
    MessageBasic,
    MessageRead,
    RunRead,
    RunReadFull,
    Success,
    TagCreate,
    TagRead,
    TagUpdate,
    UserMessage,
    InfoMessage,
    StatusMessage,
    __version__,
]



from .agents import AgentsResource as AgentsResource
from .conversations import ConversationsResource as ConversationsResource
from .runs import RunsResource as RunsResource
from .tags import TagsResource as TagsResource
